function [J grad] = nnCostFunction(nn_params, ...
                                   input_layer_size, ...
                                   hidden_layer_size, ...
                                   num_labels, ...
                                   X, y, lambda)
%Implementation of the cost function for this neural network


% Reshape nn_params back into the parameters 
% Theta1 and Theta2, the weight matrices
% for our 2 layer neural network
Theta1 = reshape(nn_params(1:hidden_layer_size * ...
    (input_layer_size + 1)), ...
    hidden_layer_size, (input_layer_size + 1));

Theta2 = reshape(nn_params((1 + (hidden_layer_size * ...
    (input_layer_size + 1))):end), ...
    num_labels, (hidden_layer_size + 1));

% Setup some useful variables
m = size(X, 1);
         

%--- Part 1: implement forward propagation and calculate cost----

% --- Forward propagation through the network ---
% Convert the classifier (y) values into matrixes 
% containing 0 and ones for the correct indexes
eye_matrix = eye(num_labels);
y_matrix = eye_matrix(y,:);

% Add a column of ones to X (the bias) 
a1 = [ones(m,1) X];

%Calculate the values in the hidden layer
z2 = Theta1 * a1';
a2 = [sigmoid(z2)]';
%Add the bias to the hidden layer
a2 = [ones(size(a2,1),1) a2];


%Calculate the values in the output layer
z3 = Theta2 * a2';
a3 = [sigmoid(z3)]';

h = a3;

%---- cost function calculation---
%Calculate the cost for the neural network with regularization

mpart1 = (- y_matrix .* log(h));
mpart2 = (1 - y_matrix) .* log(1 - h);
cost = ( (1/m) * sum(sum(mpart1 - mpart2, 2)) );

%compute regularization
regur = (lambda / (2*m) ) * ( sum(sum(Theta1(:,2:end).^2, 2)) ...
    + sum(sum(Theta2(:,2:end).^2, 2)) );

J = cost + regur;




% ------------- Backpropagation--------------------%

%compute the error term in the output layer.
d3 = a3 - y_matrix;

%Compute the error term in the hidden layer.
d2 = ( d3 * Theta2(:,2:end) ) .* sigmoidGradient(z2');

%Compute accumulated gradient
Delta1 = d2' * a1;

Delta2 = d3' * a2;

Theta1_grad_un = Delta1 * (1/m);
Theta2_grad_un = Delta2 * (1/m);

%Regularization of the gradient------------------
Theta1(:,1) = 0;
Theta2(:,1) = 0;
Theta1 = Theta1 * (lambda / m);
Theta2 = Theta2 * (lambda / m);

Theta1_grad = Theta1_grad_un + Theta1;
Theta2_grad = Theta2_grad_un + Theta2;

% -------------------------------------------------------


% Unroll gradients
grad = [Theta1_grad(:) ; Theta2_grad(:)];


end
