%=================================================
%            SLUG FLOW (PART 1)
%=================================================
% Made from "Slug_flow_1.m" 

clc
clear
close all

% ********* SENSOR DISTANCES:*********
    L16=.102+.081+.062+.082+.102;
    L25=.081+.062+.082;
    L34=.062;
% ************************************
load('scan_frequency.mat');
origfolder = cd
cd('Sensor data')
filenames = dir('*.txt');

%Chose file number from the list of files in Sensor data
%from 1 to 166
filenumber = 60
fname =filenames(filenumber).name


nhead = 40; % number of lines of header information
ncols = 6;  % number of columns in the data file

[labels,V] = readColCapData(fname,ncols,nhead);
M = length(V(:,1))

Tid = 1:M;

dt = scan_frequency(filenumber)

Tid = Tid*dt;


L1 = 1
L2 = 5000 % number of points

figure(1) % Plot Capacitance sensors - time trace

subplot(3,1,1); plot(Tid(L1:L2),V(L1:L2,1),'b-');
hold on
plot(Tid(L1:L2),V(L1:L2,6),'r-');
xlabel('Time (s)','FontSize',15); % add axis labels and plot title
ylabel('Sensor signal (0 - 10000)','FontSize',15);
title('Capacitance sensors - time trace','FontSize',14);
legend('Sensor 1','Sensor 6',1);
set(gca, 'fontsize', 12)

subplot(3,1,2); plot(Tid(L1:L2),V(L1:L2,2),'b-');  
hold on;
plot(Tid(L1:L2),V(L1:L2,5),'r-');
xlabel('Time (s)','FontSize',15);                      
ylabel('Sensor signal','FontSize',15);
legend('Sensor 2','Sensor 5',1); 
set(gca, 'fontsize', 12)

subplot(3,1,3); plot(Tid(L1:L2),V(L1:L2,3),'b-'); 
hold on;
plot(Tid(L1:L2),V(L1:L2,4),'r-');
xlabel('Time (s)','FontSize',15);                      
ylabel('Sensor signal','FontSize',15);
legend('Sensor 3','Sensor 4',1);
set(gca, 'fontsize', 12)


% *********** FFT ANALYSIS  **************************
%
% ****************************************************
for i = 1 : 6
ISE=i; %Sensor number
L = M;
sensordata(:,i) = V(:,ISE);
y(:,i) = sensordata(:,i) - mean(sensordata(:,i));

Fs = 1/dt;
NFFT = 2^nextpow2(L); % Next power of 2 from length of y
Y(:,i) = fft(y(:,i),NFFT)/L;
f = Fs/2*linspace(0,1,NFFT/2);

end

for i = 1:3
    
figure(2)   % Plot Single-sided amplitude spectrum.

sens16 = [1,6];
sens25 = [2,5];
sens34 = [3,4];
if i == 1
    sensor = sens16;
elseif i == 2
    sensor = sens25;
else
    sensor = sens34;
end

subplot(3,1,i); semilogx(f,2*abs(Y(1:NFFT/2,i)),'-r') 
hold on
semilogx(f,2*abs(Y(1:NFFT/2,7-i)),'-b')
hold off
title(['Single-Sided Amplitude Spectrum of y(t) sensor ' ...
        num2str(sensor(1,1)) ' and ' num2str(sensor(1,2))])
xlabel('Frequency (Hz)')
ylabel('|Y(f)|')
legend(['Sensor ' num2str(sensor(1,1))],['Sensor ' ...
        num2str(sensor(1,2))],1)
end

%****************************
%  PDF histogram
%****************************

for i = 1:3
figure(4)  % Histrogram Probability density function
sens16 = [1,6];
sens25 = [2,5];
sens34 = [3,4];
if i == 1
    sensor = sens16;
elseif i == 2
    sensor = sens25;
else
    sensor = sens34;
end

subplot(3,1,i); hist(sensordata(:,sensor),100)
title(['Probability density funciton sensor ' ...
        num2str(sensor(1,1)) ' and ' num2str(sensor(1,2))])
xlabel ('Sensor reading')
ylabel('Number of counts')
legend(['Sensor ' num2str(sensor(1,1))],['Sensor ' ...
        num2str(sensor(1,2))],1)
end

%*****CROSS CORRELATION FUNCTION**************
%
%*********************************************
for j = 1:3
x(:,j)=V(L1:L2,j);  % 1 function (1 sensor)
y(:,j)=V(L1:L2,7-j);  % 2 function (6 sensor)

N=length(Tid(L1:L2));
yy(j,:)=y(:,j)';
ntau=200
for i=1:ntau
yy(j,:) = circshift(yy(j,:),[0,+1]);
CCF(i,j) = yy(j,:)*x(:,j)/N;  
end


figure(5)   % Plot Cross correlation function

plot(CCF)
xlabel('Time (ms)'), 
ylabel('Correlation')
title('Cross correlation function')
legend('sensor 1&6', 'sensor 2&5', 'sensor 3&4',1)


maxccf=max(CCF(:,j))

for i=1:ntau
    if CCF(i,j)==maxccf
        tshift(1,j)=i       % Time shift
    end;
end

if j == 1
    sensor = L16;
elseif j == 2
    sensor = L25;
else
    sensor = L34;
end

Uslug(1,j)=sensor/(tshift(1,j)/(1/dt))  % Slug velocity

fin = fopen(fname,'r');
for i=1:9,  buffer = fgetl(fin);  end

Uls=str2double(buffer(16:28)) % Superficial liquid velocity
Ugs=str2double(buffer(40:52)) % Superficial gas velocity
Umix=Uls+Ugs;                 % Mixture velocity
Ratio(1,j)=Uslug(1,j)/Umix;              
end

Uslug
Ratio
Umix
dt
cd(origfolder);